/*
 * Copyright (C) 2011 Canonical, Ltd.
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * version 3.0 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Authored by Neil Jagdish Patel <neil.patel@canonical.com>
 *
 */

using GLib;

namespace Unity {

public enum CategoryRenderer
{
  VERTICAL_TILE,
  HORIZONTAL_TILE,
  LIST_TILE,
  FLOW,
  TABULAR_TILE,
  DYNAMIC;

  public static CategoryRenderer from_string (string renderer_name)
  {
    switch (renderer_name)
    {
      case "tile-vertical":
        return CategoryRenderer.VERTICAL_TILE;
      case "tile-horizontal":
        return CategoryRenderer.HORIZONTAL_TILE;
      case "tile-list":
        return CategoryRenderer.LIST_TILE;
      case "flow":
        return CategoryRenderer.FLOW;
      case "tile-tabular":
        return CategoryRenderer.TABULAR_TILE;
      case "dynamic":
        return CategoryRenderer.DYNAMIC;
      default:
        warning ("Unknown CategoryRenderer: %s", renderer_name);
        return CategoryRenderer.VERTICAL_TILE; // reserve a value for unknown?
    }
  }

  public static unowned string to_string (CategoryRenderer val)
  {
    switch (val)
    {
      case CategoryRenderer.VERTICAL_TILE:
        return "tile-vertical";
      case CategoryRenderer.HORIZONTAL_TILE:
        return "tile-horizontal";
      case CategoryRenderer.LIST_TILE:
        return "tile-list";
      case CategoryRenderer.FLOW:
        return "flow";
      case CategoryRenderer.TABULAR_TILE:
        return "tile-tabular";
      case CategoryRenderer.DYNAMIC:
        return "dynamic";
      default:
        return "unknown-renderer";
    }
  }
}

public enum CategoryContentType
{
  DEFAULT,
  APPLICATIONS,
  MUSIC,
  VIDEO,
  PLACES,
  SOCIAL;

  public static CategoryContentType from_string (string content_type)
  {
    switch (content_type)
    {
      case "apps":
        return CategoryContentType.APPLICATIONS;
      case "music":
        return CategoryContentType.MUSIC;
      case "video":
        return CategoryContentType.VIDEO;
      case "places":
        return CategoryContentType.PLACES;
      case "social":
        return CategoryContentType.SOCIAL;
      default:
        return CategoryContentType.DEFAULT;
    }
  }

  public static unowned string to_string (CategoryContentType val)
  {
    switch (val)
    {
      case CategoryContentType.APPLICATIONS:
        return "apps";
      case CategoryContentType.MUSIC:
        return "music";
      case CategoryContentType.VIDEO:
        return "video";
      case CategoryContentType.PLACES:
        return "places";
      case CategoryContentType.SOCIAL:
        return "social";
      default:
        return "default";
    }
  }
}

public class Category : GLib.Object
{
  public string id { get; construct; }
  public string name { get; construct; }
  public Icon? icon_hint { get; construct; }
  public CategoryRenderer default_renderer { get; construct; }
  public CategoryContentType content_type { get; construct set; }

  public Category (string id,
                   string name,
                   Icon icon_hint,
                   CategoryRenderer renderer=CategoryRenderer.VERTICAL_TILE)
  {
    Object (id: id, name:name, icon_hint:icon_hint, default_renderer:renderer);
  }

  /*
   * Implementation
   */
  public string renderer {
    get {
      return CategoryRenderer.to_string (default_renderer);
    }
  }

  private HashTable<string, Variant> hints;

  internal unowned HashTable<string, Variant> get_hints ()
  {
    if (hints == null)
    {
      hints = new HashTable<string, Variant> (str_hash, str_equal);
    }

    hints["content-type"] =
      new Variant.string (CategoryContentType.to_string (content_type));

    return hints;
  }
}

} /* namespace */
