/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * Copyright (C) 2011 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Michal Hruby <michal.hruby@canonical.com>
 *
 */
using Unity;
using Unity.Protocol;

namespace Unity.Test
{
  public class ScopeSuite
  {
    public ScopeSuite ()
    {
      GLib.Test.add_data_func ("/Unit/AbstractScope/AddResult",
        Fixture.create<AbstractScopeTester> (AbstractScopeTester.test_add_result));
      GLib.Test.add_data_func ("/Unit/AbstractScope/AddResultFromVariant",
        Fixture.create<AbstractScopeTester> (AbstractScopeTester.test_add_variant_result));
      /* disabled for now
      GLib.Test.add_data_func ("/Unit/Scope/Signal/NoSearchChanged",
                               test_no_search_changed);
      GLib.Test.add_data_func ("/Unit/Scope/Signal/SearchChanged",
                               test_search_changed);
      GLib.Test.add_data_func ("/Unit/Scope/Signal/SearchKey",
                               test_search_key);
      GLib.Test.add_data_func ("/Unit/Scope/Signal/SearchKeyDetails",
                               test_search_key_details);
      GLib.Test.add_data_func ("/Unit/Scope/Signals/MultipleQueueCalls",
                               test_multiple_queue_calls);
      GLib.Test.add_data_func ("/Unit/Scope/Signals/Cancellable",
                               test_cancellable);
      */
    }

    class AbstractScopeTester: Object, Fixture
    {
      delegate void RunFunc (SearchContext search);

      private AbstractScopeTestImpl scope;

      class ScopeSearchTestImpl: ScopeSearchBase
      {
        private unowned RunFunc runner;
        public ScopeSearchTestImpl (RunFunc run_func)
        {
          runner = run_func;
        }
        protected override void run ()
        {
          runner (search_context);
        }
      }

      class AbstractScopeTestImpl : AbstractScope
      {
        public RunFunc run_func;

        public AbstractScopeTestImpl ()
        {
        }

        protected override ScopeSearchBase create_search_for_query (SearchContext search_context)
        {
          var search_runner = new ScopeSearchTestImpl (run_func);
          search_runner.set_search_context (search_context);
          return search_runner;
        }

        protected override ResultPreviewer create_previewer (ScopeResult result, SearchMetadata metadata)
        {
          return null;
        }

        protected override CategorySet get_categories ()
        {
          return new CategorySet ();
        }

        protected override FilterSet get_filters ()
        {
          return new FilterSet ();
        }

        protected override Schema get_schema ()
        {
          return new Schema ();
        }

        protected override string get_group_name ()
        {
          return "com.canonical.example.Scope.Test";
        }

        protected override string get_unique_name ()
        {
          return "/com/canonical/example/Scope/Test";
        }
      }

      private void setup ()
      {
        scope = new AbstractScopeTestImpl ();
      }

      private void teardown ()
      {
      }

      class ResultSetTestImpl: ResultSet
      {
        public GenericArray<ScopeResult?> results;

        construct
        {
          results = new GenericArray<ScopeResult?> ();
        }
        protected override void add_result (ScopeResult result)
        {
          results.add (result);
        }
      }

      public void test_add_result ()
      {
        bool runner_invoked = false;
        scope.run_func = (context) =>
        {
          runner_invoked = true;
          var result = ScopeResult ();
          result.uri = "test://";
          result.icon_hint = "unknown";
          result.category = 0;
          result.mimetype = "text/html";
          result.title = "Test";
          result.comment = "comment";
          result.dnd_uri = result.uri;

          context.result_set.add_result (result);
        };

        var result_set = new ResultSetTestImpl ();
        var search = scope.create_search_for_query (
          SearchContext.create ("foo", SearchType.DEFAULT,
                                new FilterSet (), null,
                                result_set, null));
        search.run ();

        assert (runner_invoked);
        assert (result_set.results.length == 1);

        var test_result = result_set.results[0];
        assert (test_result.uri == "test://");
        assert (test_result.title == "Test");
      }
      
      public void test_add_variant_result ()
      {
        bool runner_invoked = false;
        scope.run_func = (context) =>
        {
          runner_invoked = true;
          var result = ScopeResult ();
          result.uri = "test://";
          result.icon_hint = "unknown";
          result.category = 0;
          result.mimetype = "text/html";
          result.title = "Test";
          result.comment = "comment";
          result.dnd_uri = result.uri;

          var metadata = new Variant.array (VariantType.VARDICT.element (), {});
          var variant = new Variant ("(ssuussss@a{sv})",
                                     result.uri, result.icon_hint,
                                     result.category, result.result_type,
                                     result.mimetype, result.title,
                                     result.comment, result.dnd_uri,
                                     metadata);

          context.result_set.add_result_from_variant (variant);
        };

        var result_set = new ResultSetTestImpl ();
        var search = scope.create_search_for_query (
          SearchContext.create ("foo", SearchType.DEFAULT,
                                new FilterSet (), null,
                                result_set, null));
        search.run ();

        assert (runner_invoked);
        assert (result_set.results.length == 1);

        var test_result = result_set.results[0];
        assert (test_result.uri == "test://");
        assert (test_result.title == "Test");
      }
    }

/* disabled for now
    static void test_no_search_changed ()
    {
      MainLoop ml = new MainLoop ();
      var scope = new Unity.Scope ("com/canonical/Scope/Test", "test_scope");
      assert (scope != null);

      bool got_signal = false;
      scope.search_changed.connect (() =>
      {
        got_signal = true;
        ml.quit ();
      });

      // queue_search_changed shouldn't emit the search_changed signal if
      // appropriate view isn't active
      scope.queue_search_changed (SearchType.DEFAULT);

      uint tid = Timeout.add (500, () => { ml.quit (); return false; });

      ml.run ();
      assert (got_signal == false);
      Source.remove (tid);
    }

    static void test_search_changed ()
    {
      MainLoop ml = new MainLoop ();
      var scope = new Unity.Scope ("com/canonical/Scope/Test", "test_scope");
      assert (scope != null);
      scope.set_view_type_internal (ViewType.LENS_VIEW);

      bool got_signal = false;
      scope.search_changed.connect (() =>
      {
        got_signal = true;
        ml.quit ();
      });

      scope.queue_search_changed (SearchType.DEFAULT);

      // we don't want to wait indefinitely
      uint tid = Timeout.add (500, () => { assert_not_reached (); });

      ml.run ();
      assert (got_signal == true);
      Source.remove (tid);
    }

    static void test_search_key ()
    {
      MainLoop ml = new MainLoop ();
      var scope = new Unity.Scope ("com/canonical/Scope/Test", "test_scope");
      assert (scope != null);
      scope.set_view_type_internal (ViewType.LENS_VIEW);

      bool got_changed_signal = false;
      scope.search_changed.connect ((lens_search, search_type, cancellable) =>
      {
        got_changed_signal = true;
        ml.quit ();
      });

      bool got_gen_signal = false;
      scope.generate_search_key.connect ((lens_search) =>
      {
        got_gen_signal = true;
        return "foo";
      });

      scope.queue_search_changed (SearchType.DEFAULT);

      // we don't want to wait indefinitely
      uint tid = Timeout.add (500, () => { assert_not_reached (); });

      ml.run ();
      assert (got_changed_signal == true);
      assert (got_gen_signal == true);

      Source.remove (tid);
    }
    
    static void test_search_key_details ()
    {
      MainLoop ml = new MainLoop ();
      var scope = new Unity.Scope ("com/canonical/Scope/Test", "test_scope");
      assert (scope != null);
      scope.set_view_type_internal (ViewType.LENS_VIEW);

      int num_changed_signals = 0;
      int num_gen_signals = 0;
      scope.search_changed.connect ((lens_search, search_type, cancellable) =>
      {
        num_changed_signals++;

        if (num_changed_signals == 1)
          scope.set_view_type_internal (ViewType.HOME_VIEW);
        else if (num_changed_signals >= 2) ml.quit ();
      });

      scope.generate_search_key["default"].connect ((lens_search) =>
      {
        num_gen_signals++;
        return "foo";
      });

      scope.queue_search_changed (SearchType.DEFAULT);
      scope.queue_search_changed (SearchType.GLOBAL);

      // we don't want to wait indefinitely
      uint tid = Timeout.add (500, () => { assert_not_reached (); });

      ml.run ();
      assert (num_changed_signals == 2);
      assert (num_gen_signals == 1);

      Source.remove (tid);
    }
    
    static void test_multiple_queue_calls ()
    {
      MainLoop ml = new MainLoop ();
      var scope = new Unity.Scope ("com/canonical/Scope/Test", "test_scope");
      assert (scope != null);
      scope.set_view_type_internal (ViewType.LENS_VIEW);

      int num_changed = 0;
      scope.search_changed.connect (() =>
      {
        num_changed++;
      });

      scope.queue_search_changed (SearchType.DEFAULT);
      scope.queue_search_changed (SearchType.GLOBAL);
      scope.queue_search_changed (SearchType.DEFAULT);
      scope.queue_search_changed (SearchType.GLOBAL);
      scope.queue_search_changed (SearchType.DEFAULT);
      scope.queue_search_changed (SearchType.GLOBAL);
      scope.queue_search_changed (SearchType.DEFAULT);
      scope.queue_search_changed (SearchType.DEFAULT);
      scope.queue_search_changed (SearchType.GLOBAL);
      scope.queue_search_changed (SearchType.GLOBAL);

      // we don't want to wait indefinitely
      Timeout.add (500, () => { ml.quit (); return false; });

      ml.run ();
      // we expect just one signal (for SearchType.DEFAULT, LENS_VIEW is active)
      assert (num_changed == 1);
    }

    static void test_cancellable ()
    {
      MainLoop ml = new MainLoop ();
      var scope = new Unity.Scope ("com/canonical/Scope/Test", "test_scope");
      assert (scope != null);
      scope.set_view_type_internal (ViewType.LENS_VIEW);

      Cancellable? canc = null;
      scope.search_changed.connect ((lens_search, search_type, cancellable) =>
      {
        if (canc == null)
        {
          canc = cancellable;
          scope.queue_search_changed (SearchType.DEFAULT);
        }
        else
        {
          assert (cancellable.is_cancelled () == false);
          ml.quit ();
        }
      });

      scope.queue_search_changed (SearchType.DEFAULT);

      // we don't want to wait indefinitely
      uint tid = Timeout.add (500, () => { assert_not_reached (); });

      ml.run ();
      assert (canc != null);
      assert (canc.is_cancelled () == true);

      Source.remove (tid);
    }
*/
  }
}
